import os

from azure.core.credentials import AzureKeyCredential
from azure.ai.textanalytics import TextAnalyticsClient

from textalytics_core.entity_recognition import EntityRecognizer
from textalytics_core.resources import TextInput, EntityRecognizerOutput, Entity


class AzureEntityRecognizer(EntityRecognizer):
    def recognize_entities(self, text_input: TextInput):
        entities = self.invoke_azure_detect_entities(text_input.source_text)
        return EntityRecognizerOutput(entities=entities)

    def invoke_azure_detect_entities(self, text_content):
        language = "en"
        endpoint = os.environ["AZURE_TEXT_ANALYTICS_ENDPOINT"]
        key = os.environ["AZURE_TEXT_ANALYTICS_KEY"]

        text_analytics_client = TextAnalyticsClient(endpoint=endpoint, credential=AzureKeyCredential(key))
        documents = []
        documents.append(text_content)
        azure_response = text_analytics_client.recognize_entities(documents)
        azure_response = [review for review in azure_response if not review.is_error]
        return self.extract_entities(azure_response)

    def extract_entities(self, azure_response):
        entity_data = {}
        for idx, azure_entities in enumerate(azure_response):
            for azure_entity in azure_entities.entities:
                ent_text = azure_entity.text
                entity_data.setdefault(ent_text, {})
                entity_data[ent_text]["entity"] = ent_text

                entity_data[ent_text]["label"] = azure_entity.category

                entity_data[ent_text]["start_offset"] = azure_entity.offset
                entity_data[ent_text]["end_offset"] = azure_entity.offset + azure_entity.length

        entities = []
        for key, values in entity_data.items():
            print(values)
            entities.append(Entity.parse_obj(values))
        return entities



import os

from textalytics_client.text_analysis import Client
from textalytics_core.entity_recognition import EntityRecognizer
from textalytics_core.resources import TextInput, EntityRecognizerOutput, Entity

from tests.test_config import SAMPLE_TEXT


class TextalyticsEntityRecognizer(EntityRecognizer):
    def recognize_entities(self, text_input: TextInput):
        entities = self.invoke_textalytics_detect_entities(text_input.source_text)
        return EntityRecognizerOutput(entities=entities)

    def invoke_textalytics_detect_entities(self, text_content):
        language = "en"
        url = os.environ["TEXTALYTICS_ENDPOINT"]
        username = os.environ["TEXTALYTICS_USERNAME"]
        password = os.environ["TEXTALYTICS_PASSWORD"]

        client = Client(service_base_url=url, username=username, password=password)
        text_input = TextInput(source_text=SAMPLE_TEXT, source_language=language)
        textalytics_response = client.extract_entities(text_input=text_input)
        return self.extract_entities(textalytics_response)

    def extract_entities(self, extract_entities_response):
        entities = []

        if extract_entities_response.status_code == 200:
            extracted_entities = extract_entities_response.json()
            for extracted_entity in extracted_entities["entities"]:
                entities.append(Entity.parse_obj(extracted_entity))

        return entities




# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': '.'}

packages = \
['textalytics_aws',
 'textalytics_azure',
 'textalytics_gcp',
 'textalytics_hosted']

package_data = \
{'': ['*']}

install_requires = \
['azure-ai-textanalytics>=5.1.0,<6.0.0',
 'boto3>=1.18.40,<2.0.0',
 'google-cloud-language>=2.2.2,<3.0.0',
 'pydantic>=1.8.2,<2.0.0',
 'textalytics-core>=0.1.1,<0.2.0',
 'textalytics-python-client>=0.1.0,<0.2.0']

entry_points = \
{'console_scripts': ['test = scripts:test']}

setup_kwargs = {
    'name': 'textalytics-cloud-api-adapters',
    'version': '0.1.3',
    'description': 'textalytics adapters for various cloud apis',
    'long_description': None,
    'author': 'Manoj Bharadwaj',
    'author_email': 'manoj@cloudcosmos.tech',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import boto3
from textalytics_core.entity_recognition import EntityRecognizer
from textalytics_core.resources import TextInput, EntityRecognizerOutput, Entity


class AwsEntityRecognizer(EntityRecognizer):
    def recognize_entities(self, text_input: TextInput):
        entities = self.invoke_aws_detect_entities(text_input.source_text)
        return EntityRecognizerOutput(entities=entities)

    def invoke_aws_detect_entities(self, text_content):
        language = "en"
        comprehend_client = boto3.client('comprehend')
        aws_response = comprehend_client.detect_entities(Text=text_content, LanguageCode=language)
        return self.extract_entities(aws_response)

    def extract_entities(self, aws_response):
        entity_data = {}
        for aws_entity in aws_response["Entities"]:
            ent_text = aws_entity["Text"]
            entity_data.setdefault(ent_text, {})
            entity_data[ent_text]["entity"] = ent_text

            entity_data[ent_text]["label"] = aws_entity["Type"]

            entity_data[ent_text]["start_offset"] = aws_entity["BeginOffset"]
            entity_data[ent_text]["end_offset"] = aws_entity["EndOffset"]

        entities = []
        for key, values in entity_data.items():
            print(values)
            entities.append(Entity.parse_obj(values))
        return entities



from google.cloud import language_v1
from textalytics_core.entity_recognition import EntityRecognizer
from textalytics_core.resources import TextInput, EntityRecognizerOutput, Entity


class GcpEntityRecognizer(EntityRecognizer):
    def recognize_entities(self, text_input: TextInput):
        entities = self.invoke_gcp_analyze_entities(text_input.source_text)
        return EntityRecognizerOutput(entities=entities)

    def invoke_gcp_analyze_entities(self, text_content):
        client = language_v1.LanguageServiceClient()
        type_ = language_v1.Document.Type.PLAIN_TEXT
        language = "en"
        document = {"content": text_content, "type_": type_, "language": language}
        encoding_type = language_v1.EncodingType.UTF8

        gcp_response = client.analyze_entities(request = {'document': document, 'encoding_type': encoding_type})
        return self.extract_entities(gcp_response)

    def extract_entities(self, gcp_response):
        entity_data = {}
        for gcp_entity in gcp_response.entities:
            ent_text = gcp_entity.name
            entity_data.setdefault(ent_text, {})
            entity_data[ent_text]["entity"] = ent_text

            entity_label = language_v1.Entity.Type(gcp_entity.type_).name
            entity_data[ent_text]["label"] = entity_label

            for metadata_name, metadata_value in gcp_entity.metadata.items():
                if metadata_name == "wikipedia_url":
                    entity_data[ent_text]["resolution_link"] = metadata_value
                elif metadata_name == "mid":
                    entity_data[ent_text]["resolution_id"] = metadata_value
                    entity_data[ent_text]["resolution_type"] = metadata_name

        entities = []
        for key, values in entity_data.items():
            print(values)
            entities.append(Entity.parse_obj(values))
        return entities
